<?php

namespace Drupal\graphql_compose;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Plugin\Discovery\YamlDirectoryDiscovery;
use Drupal\Core\Plugin\Factory\ContainerFactory;

/**
 * Defines a plugin manager to deal with {{ plugin_type|pluralize }}.
 *
 * Modules can define {{ plugin_type|pluralize }} in a MODULE_NAME.{{ plugin_type|pluralize }}.yml file contained
 * in the module's base directory. Each {{ plugin_type }} has the following structure:
 *
 * @code
 *   MACHINE_NAME:
 *     label: STRING
 *     description: STRING
 * @endcode
 *
 * @see \Drupal\graphql_compose\GraphQlFormatterDefault
 * @see \Drupal\graphql_compose\GraphQlFormatterInterface
 * @see plugin_api
 */
class GraphQlFormatterPluginManager extends DefaultPluginManager {

  /**
   * {@inheritdoc}
   */
  protected $defaults = [
    // The graphql_formatter id. Set by the plugin system based on the top-level YAML key.
    'id' => '',
    // The graphql_formatter label.
    'label' => '',
    // The graphql_formatter description.
    'description' => '',
    // Default plugin class.
    //'class' => 'Drupal\graphql_compose\GraphQlFormatterDefault'
  ];

  /**
   * Constructs GraphQlFormatterPluginManager object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   */
  public function __construct(ModuleHandlerInterface $module_handler, CacheBackendInterface $cache_backend) {
    $this->factory = new ContainerFactory($this);
    $this->moduleHandler = $module_handler;
    $this->alterInfo('graphql_formatter_info');
    $this->setCacheBackend($cache_backend, 'graphql_formatter_plugins');
  }

  /**
   * {@inheritdoc}
   */
  protected function getDiscovery() {
    if (!isset($this->discovery)) {
      $directories = array_map(function ($directory) {
        return [$directory . '/graphql_formatters'];
      }, $this->moduleHandler->getModuleDirectories());

      $yaml_discovery = new YamlDirectoryDiscovery($directories, 'graphql_formatter');
      //$this->discovery = new ContainerDerivativeDiscoveryDecorator($yaml_discovery);
      $this->discovery = $yaml_discovery;
    }
    return $this->discovery;
  }

}
